@@ -349,7 +349,6 @@ typedef struct TAG_MK_VM_STRUCT
 #define MK_VM_FRAME_ITEM_TYPE_CLASS				0x01000000
 #define MK_VM_FRAME_ITEM_TYPE_INT_VALUE			0x02000000
 #define MK_VM_FRAME_ITEM_TYPE_FLOAT_VALUE		0x03000000
-#define MK_VM_FRAME_ITEM_TYPE_DOUBLE_VALUE		0x04000000
 #define MK_VM_FRAME_ITEM_TYPE_STRING_VALUE		0x05000000
 #define MK_VM_FRAME_ITEM_TYPE_CHARACTER_VALUE	0x06000000
 #define MK_VM_FRAME_ITEM_TYPE_BOOL_VALUE		0x07000000
@@ -359,6 +358,15 @@ typedef struct TAG_MK_VM_STRUCT
 #define MK_VM_FRAME_ITEM_TYPE_SUPER				0x0f400000
 #define MK_VM_FRAME_ITEM_TYPE_OWNER				0x0f600000
 
+#define MK_VM_FRAME_ITEM_TYPE_READ_PUBLIC		0x00010000
+#define MK_VM_FRAME_ITEM_TYPE_READ_PROTECTED	0x00020000
+#define MK_VM_FRAME_ITEM_TYPE_READ_PRIVATE		0x00030000
+#define MK_VM_FRAME_ITEM_TYPE_WRITE_PUBLIC		0x00040000
+#define MK_VM_FRAME_ITEM_TYPE_WRITE_PROTECTED	0x00080000
+#define MK_VM_FRAME_ITEM_TYPE_WRITE_PRIVATE		0x000c0000
+
+#define MK_VM_FRAME_ITEM_TYPE_FINAL				0x00008000
+
 // MK_VM_FRAME_ITEM_PTR value
 #define MK_VM_FRAME_ITEM_TYPE_DIRECT_VALUE		0x00000001
 #define MK_VM_FRAME_ITEM_TYPE_DIRECT_MASK		0x70000000
@@ -367,8 +375,6 @@ typedef struct TAG_MK_VM_STRUCT
 #define MK_VM_FRAME_ITEM_TYPE_DIRECT_CHAR		0x20000000
 #define MK_VM_FRAME_ITEM_TYPE_INT_MINUS_VALUE	0x80000000
 #define MK_VM_FRAME_ITEM_DIRECT_INT_VALUE_MASK	0x0ffffff0	// -16777215 ~ +16777215
-#define MK_VM_FRAME_ITEM_DIRECT_BOOL_TRUE		0x00000010	// bool : true
-#define MK_VM_FRAME_ITEM_DIRECT_BOOL_FALSE		0x00000000	// bool : true
 
 // attribute of MK_TYPE_CLASS
 
@@ -420,7 +426,7 @@ typedef struct TAG_MK_VM_STRUCT
 #define MK_TYPE_NODE_EXPR_SYMBOL			0x02000000
 #define MK_TYPE_NODE_EXPR_SYMBOL_THIS		0x02200000
 #define MK_TYPE_NODE_EXPR_SYMBOL_OWNER		0x02400000
-#define MK_TYPE_NODE_EXPR_SYMBOL_SUPER		0x02600000	// not implemented yet
+#define MK_TYPE_NODE_EXPR_SYMBOL_SUPER		0x02600000
 #define MK_TYPE_NODE_EXPR_SYMBOL_NIL		0x02800000	// not implemented yet
 #define MK_TYPE_NODE_EXPR_SYMBOL_TRUE		0x02a00000	// not implemented yet
 #define MK_TYPE_NODE_EXPR_SYMBOL_FALSE		0x02c00000	// not implemented yet
@@ -428,6 +434,7 @@ typedef struct TAG_MK_VM_STRUCT
 #define MK_TYPE_NODE_EXPR_DBLATSYMBOL		0x04000000
 #define MK_TYPE_NODE_EXPR_MULTIPLESYMBOL	0x05000000
 #define MK_TYPE_NODE_EXPR_FUNCTION_CALL		0x06000000
+#define MK_TYPE_NODE_EXPR_SUPER_CALL		0x06600000	
 #define MK_TYPE_NODE_EXPR_RETURN			0x07000000
 #define MK_TYPE_NODE_EXPR_BREAK				0x08000000
 #define MK_TYPE_NODE_EXPR_CONTINUE			0x09000000
@@ -578,32 +585,23 @@ typedef struct TAG_MK_VM_STRUCT
 #define MK_LEX_TYPE_RESERVED_SYMBOL_ELSEIF	MK_LEX_TYPE_RESERVED_SYMBOL + 9
 #define MK_LEX_TYPE_RESERVED_SYMBOL_ELSE	MK_LEX_TYPE_RESERVED_SYMBOL + 10
 #define MK_LEX_TYPE_RESERVED_SYMBOL_WHILE	MK_LEX_TYPE_RESERVED_SYMBOL + 11
-#define MK_LEX_TYPE_RESERVED_SYMBOL_FOR		MK_LEX_TYPE_RESERVED_SYMBOL + 12
-#define MK_LEX_TYPE_RESERVED_SYMBOL_BREAK	MK_LEX_TYPE_RESERVED_SYMBOL + 13
-#define MK_LEX_TYPE_RESERVED_SYMBOL_CONTINUE	MK_LEX_TYPE_RESERVED_SYMBOL + 14
-#define MK_LEX_TYPE_RESERVED_SYMBOL_DO		MK_LEX_TYPE_RESERVED_SYMBOL + 15
-#define MK_LEX_TYPE_RESERVED_SYMBOL_NEW		MK_LEX_TYPE_RESERVED_SYMBOL + 16
-#define MK_LEX_TYPE_RESERVED_SYMBOL_DELETE	MK_LEX_TYPE_RESERVED_SYMBOL + 17
-#define MK_LEX_TYPE_RESERVED_SYMBOL_NIL		MK_LEX_TYPE_RESERVED_SYMBOL + 18
-#define MK_LEX_TYPE_RESERVED_SYMBOL_TRUE	MK_LEX_TYPE_RESERVED_SYMBOL + 19
-#define MK_LEX_TYPE_RESERVED_SYMBOL_FALSE	MK_LEX_TYPE_RESERVED_SYMBOL + 20
-#define MK_LEX_TYPE_RESERVED_SYMBOL_EXPORT	MK_LEX_TYPE_RESERVED_SYMBOL + 21
-#define MK_LEX_TYPE_RESERVED_SYMBOL_REF		MK_LEX_TYPE_RESERVED_SYMBOL + 22
-#define MK_LEX_TYPE_RESERVED_SYMBOL_INT8	MK_LEX_TYPE_RESERVED_SYMBOL + 23
-#define MK_LEX_TYPE_RESERVED_SYMBOL_INT16	MK_LEX_TYPE_RESERVED_SYMBOL + 24
-#define MK_LEX_TYPE_RESERVED_SYMBOL_INT32	MK_LEX_TYPE_RESERVED_SYMBOL + 25
-#define MK_LEX_TYPE_RESERVED_SYMBOL_INT64	MK_LEX_TYPE_RESERVED_SYMBOL + 26
-#define MK_LEX_TYPE_RESERVED_SYMBOL_RAISE	MK_LEX_TYPE_RESERVED_SYMBOL + 27
-#define MK_LEX_TYPE_RESERVED_SYMBOL_TRY	MK_LEX_TYPE_RESERVED_SYMBOL + 28
-#define MK_LEX_TYPE_RESERVED_SYMBOL_CATCH	MK_LEX_TYPE_RESERVED_SYMBOL + 29
-#define MK_LEX_TYPE_RESERVED_SYMBOL_FINALLY	MK_LEX_TYPE_RESERVED_SYMBOL + 30
-#define MK_LEX_TYPE_RESERVED_SYMBOL_RETRY MK_LEX_TYPE_RESERVED_SYMBOL + 31
-#define MK_LEX_TYPE_RESERVED_SYMBOL_PUBLIC	MK_LEX_TYPE_RESERVED_SYMBOL + 32
-#define MK_LEX_TYPE_RESERVED_SYMBOL_PROTECTED	MK_LEX_TYPE_RESERVED_SYMBOL + 33
-#define MK_LEX_TYPE_RESERVED_SYMBOL_PRIVATE		MK_LEX_TYPE_RESERVED_SYMBOL + 34
-#define MK_LEX_TYPE_RESERVED_SYMBOL_READ		MK_LEX_TYPE_RESERVED_SYMBOL + 35
-#define MK_LEX_TYPE_RESERVED_SYMBOL_WRITE		MK_LEX_TYPE_RESERVED_SYMBOL + 36
-
+#define MK_LEX_TYPE_RESERVED_SYMBOL_BREAK	MK_LEX_TYPE_RESERVED_SYMBOL + 12
+#define MK_LEX_TYPE_RESERVED_SYMBOL_CONTINUE	MK_LEX_TYPE_RESERVED_SYMBOL + 13
+#define MK_LEX_TYPE_RESERVED_SYMBOL_DO		MK_LEX_TYPE_RESERVED_SYMBOL + 14
+#define MK_LEX_TYPE_RESERVED_SYMBOL_NEW		MK_LEX_TYPE_RESERVED_SYMBOL + 15
+#define MK_LEX_TYPE_RESERVED_SYMBOL_NIL		MK_LEX_TYPE_RESERVED_SYMBOL + 16
+#define MK_LEX_TYPE_RESERVED_SYMBOL_TRUE	MK_LEX_TYPE_RESERVED_SYMBOL + 17
+#define MK_LEX_TYPE_RESERVED_SYMBOL_FALSE	MK_LEX_TYPE_RESERVED_SYMBOL + 18
+#define MK_LEX_TYPE_RESERVED_SYMBOL_RAISE	MK_LEX_TYPE_RESERVED_SYMBOL + 19
+#define MK_LEX_TYPE_RESERVED_SYMBOL_TRY	MK_LEX_TYPE_RESERVED_SYMBOL + 20
+#define MK_LEX_TYPE_RESERVED_SYMBOL_CATCH	MK_LEX_TYPE_RESERVED_SYMBOL + 21
+#define MK_LEX_TYPE_RESERVED_SYMBOL_FINALLY	MK_LEX_TYPE_RESERVED_SYMBOL + 22
+#define MK_LEX_TYPE_RESERVED_SYMBOL_PUBLIC	MK_LEX_TYPE_RESERVED_SYMBOL + 23
+#define MK_LEX_TYPE_RESERVED_SYMBOL_PROTECTED	MK_LEX_TYPE_RESERVED_SYMBOL + 24
+#define MK_LEX_TYPE_RESERVED_SYMBOL_PRIVATE		MK_LEX_TYPE_RESERVED_SYMBOL + 25
+#define MK_LEX_TYPE_RESERVED_SYMBOL_READ		MK_LEX_TYPE_RESERVED_SYMBOL + 26
+#define MK_LEX_TYPE_RESERVED_SYMBOL_WRITE		MK_LEX_TYPE_RESERVED_SYMBOL + 27
+#define MK_LEX_TYPE_RESERVED_SYMBOL_INITIALIZE	MK_LEX_TYPE_RESERVED_SYMBOL + 28
 
 #define MK_LEX_STRING_MARK					'\"'
 #define MK_LEX_CHARACTER_MARK				'\''
@@ -690,10 +688,11 @@ MK_VARIABLE *mk_create_variable( MK_VM_STRUCT *vm, MK_CHAR *mk_name, unsigned in
 // vm
 int mk_vm_initialize( MK_VM_STRUCT *vm );
 int mk_vm_run( MK_VM_STRUCT *vm );
-unsigned int mk_native_function_call( MK_VM_STRUCT *vm, MK_VM_FRAME_ITEM **result, MK_FUNCTION *pFunction, unsigned int sizeParam );
+unsigned int mk_native_function_call( MK_VM_STRUCT *vm, MK_FUNCTION *pFunction, unsigned int sizeParam );
 int mk_vm_frame_item_to_int32( MK_VM_FRAME_ITEM *target );
 MK_FLOAT mk_vm_frame_item_to_float( MK_VM_FRAME_ITEM *target );
 MK_VM_FRAME_ITEM *mk_vm_create_int32_frame_item( MK_VM_STRUCT *vm, int value );
+MK_VM_FRAME_ITEM *mk_vm_create_float_frame_item( MK_VM_STRUCT *vm, MK_FLOAT value );
 MK_VM_FRAME_ITEM *mk_vm_create_bool_frame_item( int isTrue );
 int mk_vm_frame_item_is_true( MK_VM_FRAME_ITEM *target );
 #ifdef _DEBUG
@@ -789,6 +788,7 @@ int mk_gc_run( MK_VM_STRUCT *vm, int force );
 #define CLASS_INTERNAL_OBJECT	"Object"
 #define CLASS_INTERNAL_NULL		"Null"
 #define CLASS_INTERNAL_INTEGER	"Integer"
+#define	CLASS_INTERNAL_FLOAT	"Float"
 #define CLASS_INTERNAL_IO		"IO"
 #define CLASS_INTERNAL_CONSOLE	"Console"
 #define CLASS_INTERNAL_EXCEPTION "Exception"
