@@ -28,7 +28,7 @@
 #define INT16		short
 #define INT32		int
 #define INT64		long long
-#define INT_PTR		int
+#define INT_PTR		int			// 32bit
 
 
 // table size
