@@ -110,6 +110,17 @@ int mk_register_class( MK_VM_STRUCT *vm, MK_CLASS *target )
 		target ) != 0xffffffff;
 }
 
+MK_VARIABLE *mk_create_variable( MK_VM_STRUCT *vm, MK_CHAR *mk_name, unsigned int type, MK_NODE_EXPR *defaultValue )
+{
+	MK_VARIABLE *result = NULL;
+	if( MK_OBJECT_TYPE( type ) != MK_TYPE_VARIABLE )
+		return NULL;
+	result = mk_create_object( type );
+	result->name = mk_get_symbol_name_ptr( vm, mk_name );
+	result->defaultValue = defaultValue;
+	return result;
+}
+
 int mk_register_variable( MK_VM_STRUCT *vm, MK_VARIABLE *targetVariable, MK_CLASS *targetClass )
 {
 	MK_CHAR *name = 
@@ -122,6 +133,21 @@ int mk_register_variable( MK_VM_STRUCT *vm, MK_VARIABLE *targetVariable, MK_CLAS
 	return mk_insert_item_hashtable( targetClass->variables, name, targetVariable ) != 0xffffffff;
 }
 
+MK_FUNCTION *mk_create_native_function( MK_VM_STRUCT *vm, MK_CHAR *mk_name, unsigned int sizeArgs, unsigned int type, void *fp )
+{
+	MK_FUNCTION *result = NULL;
+
+	if( MK_OBJECT_TYPE( type ) != MK_TYPE_FUNCTION )
+		return NULL;
+	type |= 
+		( MK_TYPE_ATTRIBUTE_FUNCTION_NATIVE );
+	result = mk_create_object( type );
+	result->name = mk_get_symbol_name_ptr( vm, mk_name );
+	result->entryPoint = fp;
+	result->sizeArgs = sizeArgs;
+	return result;
+}
+
 int mk_register_function( MK_VM_STRUCT *vm, MK_FUNCTION *targetFunction, MK_CLASS *targetClass )
 {
 	MK_CHAR *name = 
@@ -1129,7 +1155,17 @@ MK_NODE_EXPR *do_compile_symbol( MK_VM_STRUCT *vm, MK_LEXER *lexer, unsigned int
 					break;
 
 				case MK_LEX_TYPE_RESERVED_SYMBOL_SUPER:
-					current = create_super_expr( );
+					if( lexer->nextValue == MK_LEX_TYPE_RESERVED_MARK_PARENTHESIS )
+					{
+						current = do_compile_function_call( vm, lexer );
+						if( current != NULL )
+							current->flags = 
+								MK_TYPE_NODE_EXPR | MK_TYPE_NODE_EXPR_SUPER_CALL;
+					}
+					else
+					{
+						current = create_super_expr( );
+					}
 					break;
 
 				case MK_LEX_TYPE_RESERVED_SYMBOL_OWNER:
@@ -1587,7 +1623,7 @@ int do_compile_function( MK_VM_STRUCT *vm, MK_LEXER *lexer, MK_CLASS *targetClas
 			mk_get_token( lexer );
 			break;
 
-		case MK_LEX_TYPE_RESERVED_SYMBOL_THIS:
+		case MK_LEX_TYPE_RESERVED_SYMBOL_INITIALIZE:
 			newFunction->flags |= MK_TYPE_ATTRIBUTE_FUNCTION_CONSTRUCTER;
 			break;
 
@@ -1596,7 +1632,10 @@ int do_compile_function( MK_VM_STRUCT *vm, MK_LEXER *lexer, MK_CLASS *targetClas
 
 		default:
 			if( vm->exceptionObject == NULL )
-				mk_raise_internal_error( vm, lexer->stream.name, lexer->stream.line, MK_ERROR_TYPE_COMILE_ERROR | 2, lexer->text, "[class|this|~this|<symbol name>]", NULL );
+				mk_raise_internal_error( vm, 
+					lexer->stream.name, lexer->stream.line, 
+					MK_ERROR_TYPE_COMILE_ERROR | 2, 
+					lexer->text, "[class|initialize|<symbol name>]", NULL );
 			isError = 1;
 			break;
 		}
