@@ -3,9 +3,9 @@
 
 void mk_trace( unsigned int code, MK_CHAR *message )
 {
-	printf( "[%08x]", code );
-	printf( message );
-	printf( "\n" );
+	fprintf( stdout, "[%08x]", code );
+	fputs( message, stdout );
+	fputs( "\n", stdout );
 }
 
 void *mk_open( void*value )
