@@ -247,46 +247,19 @@ void *mk_create_object( unsigned int nsType )
 	return result;
 }
 
-MK_VARIABLE *mk_create_variable( MK_VM_STRUCT *vm, MK_CHAR *mk_name, unsigned int type, MK_NODE_EXPR *defaultValue )
-{
-	MK_VARIABLE *result = NULL;
-	if( MK_OBJECT_TYPE( type ) != MK_TYPE_VARIABLE )
-		return NULL;
-	result = mk_create_object( type );
-	result->name = mk_get_symbol_name_ptr( vm, mk_name );
-	result->defaultValue = defaultValue;
-	return result;
-}
-
-MK_FUNCTION *mk_create_native_function( MK_VM_STRUCT *vm, MK_CHAR *mk_name, unsigned int sizeArgs, unsigned int type, void *fp )
-{
-	MK_FUNCTION *result = NULL;
-
-	if( MK_OBJECT_TYPE( type ) != MK_TYPE_FUNCTION )
-		return NULL;
-	type |= 
-		( MK_TYPE_ATTRIBUTE_FUNCTION_NATIVE );
-	result = mk_create_object( type );
-	result->name = mk_get_symbol_name_ptr( vm, mk_name );
-	result->entryPoint = fp;
-	result->sizeArgs = sizeArgs;
-	return result;
-}
-
 MK_VM_FRAME_ITEM *mk_vm_create_bool_frame_item( int isTrue )
 {
-	unsigned int intResult = 0;
-	intResult = MK_VM_FRAME_ITEM_TYPE_DIRECT_VALUE | MK_VM_FRAME_ITEM_TYPE_DIRECT_BOOL;
-	if( isTrue )
-		intResult |= 0x00000010;
-	return (MK_VM_FRAME_ITEM*)intResult;
+	return ( MK_VM_FRAME_ITEM * )
+		( ( isTrue ) ? 
+			MK_VM_FRAME_ITEM_TYPE_DIRECT_VALUE | MK_VM_FRAME_ITEM_TYPE_DIRECT_BOOL | 0x00000010:
+			MK_VM_FRAME_ITEM_TYPE_DIRECT_VALUE | MK_VM_FRAME_ITEM_TYPE_DIRECT_BOOL );
 }
 
 int mk_vm_frame_item_is_true( MK_VM_FRAME_ITEM *target )
 {
 	if( (INT_PTR)target & MK_VM_FRAME_ITEM_TYPE_DIRECT_VALUE )
 	{
-		return mk_vm_frame_item_to_int32( target ) != 0;
+		return ( (INT_PTR)target & MK_VM_FRAME_ITEM_DIRECT_INT_VALUE_MASK );
 	}
 	else
 	{
@@ -296,12 +269,11 @@ int mk_vm_frame_item_is_true( MK_VM_FRAME_ITEM *target )
 			return 0;
 
 		case MK_VM_FRAME_ITEM_TYPE_INT_VALUE:
-		case MK_VM_FRAME_ITEM_TYPE_FLOAT_VALUE:
-		case MK_VM_FRAME_ITEM_TYPE_DOUBLE_VALUE:
-			return mk_vm_frame_item_to_int32( target ) != 0;
-
 		case MK_VM_FRAME_ITEM_TYPE_BOOL_VALUE:
 			return target->int32TypeValue != 0;
+
+		case MK_VM_FRAME_ITEM_TYPE_FLOAT_VALUE:
+			return target->floatTypeValue != 0;
 		
 		default:
 			return 1;
@@ -309,14 +281,24 @@ int mk_vm_frame_item_is_true( MK_VM_FRAME_ITEM *target )
 	}
 }
 
+MK_VM_FRAME_ITEM *mk_vm_create_float_frame_item( MK_VM_STRUCT *vm, MK_FLOAT value )
+{
+	MK_VM_FRAME_ITEM *result = NULL;
+	result = mk_create_vm_frame_item_object( &vm->pFrameItemTable );
+	result->flags = 
+		MK_TYPE_SET_ATTRIBUTE( result->flags, MK_VM_FRAME_ITEM_TYPE_FLOAT_VALUE );
+	result->floatTypeValue = value;
+	return result;
+}
+
 MK_VM_FRAME_ITEM *mk_vm_create_int32_frame_item( MK_VM_STRUCT *vm, int value )
 {
 	MK_VM_FRAME_ITEM *result = NULL;
 	if( abs( value ) < 0x01000000 )
 	{
 		unsigned int intResult = 0;
-		intResult = MK_VM_FRAME_ITEM_TYPE_DIRECT_VALUE | MK_VM_FRAME_ITEM_TYPE_DIRECT_INT;
-		intResult |= abs( value ) << 4;
+		intResult = 
+			MK_VM_FRAME_ITEM_TYPE_DIRECT_VALUE | MK_VM_FRAME_ITEM_TYPE_DIRECT_INT | ( abs( value ) << 4 );
 		if( value < 0 )
 			intResult |= MK_VM_FRAME_ITEM_TYPE_INT_MINUS_VALUE;
 		result = ( MK_VM_FRAME_ITEM* )intResult;
@@ -336,8 +318,8 @@ MK_FLOAT mk_vm_frame_item_to_float( MK_VM_FRAME_ITEM *target )
 	MK_FLOAT num = 0.0;
 	if( (INT_PTR)target & MK_VM_FRAME_ITEM_TYPE_DIRECT_VALUE )
 	{
-		num = (MK_FLOAT)(( target->flags & MK_VM_FRAME_ITEM_DIRECT_INT_VALUE_MASK ) >> 4);
-		if( target->flags & MK_VM_FRAME_ITEM_TYPE_INT_MINUS_VALUE )
+		num = ( MK_FLOAT )( ( (INT_PTR)target & MK_VM_FRAME_ITEM_DIRECT_INT_VALUE_MASK ) >> 4 );
+		if( (INT_PTR)target & MK_VM_FRAME_ITEM_TYPE_INT_MINUS_VALUE )
 			num *= -1.0;
 	}
 	else
@@ -351,10 +333,6 @@ MK_FLOAT mk_vm_frame_item_to_float( MK_VM_FRAME_ITEM *target )
 		case MK_VM_FRAME_ITEM_TYPE_FLOAT_VALUE:
 			num = target->floatTypeValue;
 			break;
-
-		case MK_VM_FRAME_ITEM_TYPE_DOUBLE_VALUE:
-			num = (int)target->floatTypeValue;
-			break;
 		}
 	}
 	return num;
@@ -380,10 +358,6 @@ int mk_vm_frame_item_to_int32( MK_VM_FRAME_ITEM *target )
 		case MK_VM_FRAME_ITEM_TYPE_FLOAT_VALUE:
 			result = (int)target->floatTypeValue;
 			break;
-
-		case MK_VM_FRAME_ITEM_TYPE_DOUBLE_VALUE:
-			result = (int)target->floatTypeValue;
-			break;
 		}
 	}
 	return result;
@@ -460,6 +434,7 @@ void mk_destroy_expr( MK_MEMORY_POOL *top, MK_NODE_EXPR *target )
 		{
 		case MK_TYPE_NODE_EXPR_FUNCTION_CALL:
 		case MK_TYPE_NODE_EXPR_NEW:
+		case MK_TYPE_NODE_EXPR_SUPER_CALL:
 			free_if_not_memory_pool( target->u1.symbolName );
 			mk_destroy_vector_node( top, target->u2.args );
 			break;
