@@ -29,6 +29,8 @@ static
 int mk_execute_expr_new( MK_VM_STRUCT *vm, MK_NODE_EXPR *pExpr, int hasParent );
 static
 int mk_execute_expr_call_function( MK_VM_STRUCT *vm, MK_NODE_EXPR *pExpr, int hasParent );
+static
+int mk_execute_expr_super_call( MK_VM_STRUCT *vm, MK_NODE_EXPR *pExpr, int hasParent );
 static 
 int mk_execute_expr_return( MK_VM_STRUCT *vm, MK_NODE_EXPR *pExpr, int hasParent );
 static 
@@ -59,7 +61,7 @@ int mk_execute_block_try( MK_VM_STRUCT *vm, unsigned int *target );
 void mk_vm_push_stack( MK_VECTOR *stack, MK_VM_FRAME_ITEM *target )
 {
 	mk_push_vector( stack, target );
-	printf( "push:% 3d:", stack ? stack->used : 0 );
+	fprintf( stdout, "push:% 3d:", stack ? stack->used : 0 );
 	dump_node(target, 0 );
 }
 
@@ -67,9 +69,9 @@ MK_VM_FRAME_ITEM* mk_vm_pop_stack( MK_VECTOR *stack )
 {
 	void *result = NULL;
 	if( mk_size_vector( stack ) == 0 )
-		printf( "err :stack underflow!!\n" );
+		fprintf( stdout, "err :stack underflow!!\n" );
 	result = mk_pop_vector( stack );
-	printf( "pop :% 3d:", stack->used );
+	fprintf( stdout, "pop :% 3d:", stack->used );
 	dump_node( result, 0 );
 	return result;
 }
@@ -77,6 +79,7 @@ MK_VM_FRAME_ITEM* mk_vm_pop_stack( MK_VECTOR *stack )
 
 extern MK_CLASS *mk_create_kernel_class( MK_VM_STRUCT *vm );
 extern MK_CLASS *mk_create_int_class( MK_VM_STRUCT *vm );
+extern MK_CLASS *mk_create_float_class( MK_VM_STRUCT *vm );
 extern MK_CLASS *mk_create_null_class( MK_VM_STRUCT *vm );
 extern MK_CLASS *mk_create_object_class( MK_VM_STRUCT *vm );
 extern MK_CLASS *mk_create_io_class( MK_VM_STRUCT *vm );
@@ -110,6 +113,9 @@ void mk_register_internal_classes( MK_VM_STRUCT *vm )
 
 	target = mk_create_int_class( vm );
 	mk_insert_item_hashtable( vm->code->classes, target->nameThis, target );
+	
+	target = mk_create_float_class( vm );
+	mk_insert_item_hashtable( vm->code->classes, target->nameThis, target );
 
 	target = mk_create_io_class( vm );
 	mk_insert_item_hashtable( vm->code->classes, target->nameThis, target );
@@ -226,6 +232,8 @@ int mk_vm_initialize( MK_VM_STRUCT *vm )
 		mk_execute_expr_new;
 	vm->callNodeExpr[MK_TYPE_ATTRIBUTE_INDEX(MK_TYPE_NODE_EXPR_FUNCTION_CALL)] =
 		mk_execute_expr_call_function;
+	vm->callNodeExpr[MK_TYPE_ATTRIBUTE_INDEX(MK_TYPE_NODE_EXPR_SUPER_CALL)] =
+		mk_execute_expr_super_call;
 
 	return MK_VM_EXECUTE_EXPR_RETURN_NEXTSTEP;
 }
@@ -334,7 +342,7 @@ int mk_register_global_instance( MK_VM_STRUCT *vm )
 			vm->global, 
 			key, 
 			newVariable );
-		newVariable->flags |= MK_VM_FRAME_ITEM_TYPE_CLASS;
+		newVariable->flags |= MK_VM_FRAME_ITEM_TYPE_CLASS | MK_VM_FRAME_ITEM_TYPE_FINAL;
 		newVariable->classTypeValue.typeName = mk_get_symbol_name_ptr( vm, key );
 		newVariable->classTypeValue.variables =
 			mk_allocate_vm_managed_hashtable( &vm->pHashTable );
@@ -463,7 +471,10 @@ MK_CLASS *mk_vm_get_class( MK_VM_STRUCT *vm, MK_VM_FRAME_ITEM *pTarget )
 	switch( attribute )
 	{
 	case MK_VM_FRAME_ITEM_TYPE_CLASS:
-		result = mk_vm_get_class_by_name( vm, &pTarget->classTypeValue.typeName );
+		mk_find_item_hashtable( 
+			vm->code->classes, 
+			pTarget->classTypeValue.typeName, 
+			(void**)&result );
 		break;
 
 	case MK_VM_FRAME_ITEM_TYPE_REFERENCE_VALUE:
@@ -495,7 +506,6 @@ MK_CLASS *mk_vm_get_class( MK_VM_STRUCT *vm, MK_VM_FRAME_ITEM *pTarget )
 		vm->cache->pConstantClass[index] = result;
 		break;
 
-	case MK_VM_FRAME_ITEM_TYPE_DOUBLE_VALUE:
 	case MK_VM_FRAME_ITEM_TYPE_FLOAT_VALUE:
 		mk_find_item_hashtable( classes, mk_get_symbol_name_ptr( vm, "Float" ), (void**)&result );
 		vm->cache->pConstantClass[index] = result;
@@ -926,7 +936,7 @@ MK_FUNCTION* mk_vm_find_function( MK_VM_STRUCT *vm, MK_CLASS *pOwner, const MK_C
 	return pResult;
 }
 
-int mk_mk_function_call( MK_VM_STRUCT *vm, MK_VM_FRAME_ITEM **result, MK_FUNCTION *pFunction )
+int mk_mk_function_call( MK_VM_STRUCT *vm, MK_FUNCTION *pFunction )
 {
 	int retCode = MK_VM_EXECUTE_EXPR_RETURN_FAILED;
 	MK_VM_FRAME *newFrame =
@@ -943,18 +953,20 @@ int mk_mk_function_call( MK_VM_STRUCT *vm, MK_VM_FRAME_ITEM **result, MK_FUNCTIO
 
 	// args
 	if( size > 0 )
+	{
 		newFrame->localVariables = 
 			mk_allocate_vm_managed_hashtable( &vm->pHashTable );
-	for( index = size - 1; index >= 0; index -- )
-	{
-		MK_VARIABLE *pArg = 
-			mk_get_at_vector( pFunction->args, (unsigned int)index );
-		value = mk_vm_pop_stack( params );
-		// insert local variables hashtable
-		mk_insert_item_hashtable(
-			newFrame->localVariables,
-			pArg->name,
-			value );
+		for( index = size - 1; index >= 0; index -- )
+		{
+			MK_VARIABLE *pArg = 
+				mk_get_at_vector( pFunction->args, (unsigned int)index );
+			value = mk_vm_pop_stack( params );
+			// insert local variables hashtable
+			mk_insert_item_hashtable(
+				newFrame->localVariables,
+				pArg->name,
+				value );
+		}
 	}
 
 	// ptr [this] 
@@ -964,8 +976,6 @@ int mk_mk_function_call( MK_VM_STRUCT *vm, MK_VM_FRAME_ITEM **result, MK_FUNCTIO
 	newFrame->pFunction = pFunction;
 
 	retCode = mk_execute_block( vm, pFunction->impl );
-	if( retCode == MK_VM_EXECUTE_EXPR_RETURN_RETURN )
-		*result = mk_vm_pop_stack( vm->localStack );	// set return code.
 	vm->pCurrentFrame = vm->pCurrentFrame->previous;
 	return retCode;
 }
@@ -979,27 +989,43 @@ int mk_vm_call_function( MK_VM_STRUCT *vm, MK_FUNCTION *pFunction )
 	{
 		retCode = mk_native_function_call( 
 			vm,
-			&result, 
 			pFunction, pFunction->sizeArgs );
 	}
 	else
 	{
 		retCode = mk_mk_function_call(
 			vm,
-			&result,
 			pFunction );
 	}
 	if( retCode == MK_VM_EXECUTE_EXPR_RETURN_RETURN )
-	{
-		if( result == NULL )
-			result = mk_create_vm_frame_item_object(&vm->pFrameItemTable);
-		mk_vm_push_stack( vm->localStack, result );
 		retCode = MK_VM_EXECUTE_EXPR_RETURN_NEXTSTEP;
-	}
-
 	return retCode;
 }
 
+static
+MK_VM_FRAME_ITEM *mk_vm_get_super_class_instance( MK_VM_STRUCT *vm, MK_VM_FRAME_ITEM *pTarget )
+{
+	MK_VM_FRAME_ITEM *result = NULL;
+	if( (INT_PTR)pTarget & MK_VM_FRAME_ITEM_TYPE_DIRECT_VALUE ||
+		MK_TYPE_ATTRIBUTE( pTarget->flags ) != MK_VM_FRAME_ITEM_TYPE_CLASS )
+	{
+		result = 
+			mk_create_vm_frame_item_object( &vm->pFrameItemTable );
+		result->flags = 
+			MK_TYPE_SET_ATTRIBUTE( result->flags, MK_VM_FRAME_ITEM_TYPE_CLASS );
+		result->classTypeValue.typeName = 
+			mk_get_symbol_name_ptr( vm, "Object" );
+		result->classTypeValue.child = pTarget;
+	}
+	else
+	{
+		mk_find_item_hashtable( pTarget->classTypeValue.variables,
+			mk_get_symbol_name_ptr( vm, "super" ), 
+			(void**)&result );
+	}
+	return result;
+}
+
 static
 int mk_raise_invalidate_parent_error( MK_VM_STRUCT *vm )
 {
@@ -1011,7 +1037,7 @@ int mk_execute_expr_raise( MK_VM_STRUCT *vm, MK_NODE_EXPR *pExpr, int hasParam )
 {
 	int retCode = mk_execute_expr( vm, pExpr->u1.left, 0 );
 #ifdef _DEBUG
-	printf( "raise\n" );
+	fprintf( stdout, "raise\n" );
 #endif
 	if( retCode == MK_VM_EXECUTE_EXPR_RETURN_NEXTSTEP )
 	{
@@ -1040,7 +1066,7 @@ int mk_execute_expr_operation( MK_VM_STRUCT *vm, MK_NODE_EXPR *pExpr, int hasPar
 	MK_FUNCTION *pFunction = NULL;
 
 #ifdef _DEBUG
-	printf( "call %s\n", operation_int_to_string( pExpr->flags ) );
+	fprintf( stdout, "call %s\n", operation_int_to_string( pExpr->flags ) );
 #endif
 	if( hasParent != 0 )
 		return mk_raise_invalidate_parent_error( vm );
@@ -1076,7 +1102,7 @@ int mk_execute_expr_owner( MK_VM_STRUCT *vm, MK_NODE_EXPR *pExpr, int hasParent
 	MK_VM_FRAME_ITEM *pResult = 
 		mk_create_vm_frame_item_object( &vm->pFrameItemTable );
 #ifdef _DEBUG
-	printf( "owner\n" );
+	fprintf( stdout, "owner\n" );
 #endif
 	if( hasParent != 0 )
 		return mk_raise_invalidate_parent_error( vm );
@@ -1092,7 +1118,7 @@ int mk_execute_expr_this( MK_VM_STRUCT *vm, MK_NODE_EXPR *pExpr, int hasParent )
 {
 	MK_VM_FRAME_ITEM *pResult = NULL;
 #ifdef _DEBUG
-	printf( "this\n" );
+	fprintf( stdout, "this\n" );
 #endif
 	if( hasParent == 0 )
 		mk_vm_push_stack( vm->localStack, vm->pCurrentFrame->pThis );
@@ -1103,7 +1129,7 @@ int mk_execute_expr_super( MK_VM_STRUCT *vm, MK_NODE_EXPR *pExpr, int hasParent
 {
 	MK_VM_FRAME_ITEM *pThis = NULL, *pSuper = NULL;
 #ifdef _DEBUG
-	printf( "super\n" );
+	fprintf( stdout, "super\n" );
 #endif
 	if( hasParent == 0 )
 		pThis = vm->pCurrentFrame->pThis;
@@ -1139,7 +1165,7 @@ int mk_execute_expr_constant( MK_VM_STRUCT *vm, MK_NODE_EXPR *pExpr, int hasPare
 	MK_VM_FRAME_ITEM *result = NULL;
 
 #ifdef _DEBUG
-	printf( "loadconst %08x-%08x\n", pExpr->u1.constantValue, pExpr->u2.constantValue );
+	fprintf( stdout, "loadconst %08x-%08x\n", pExpr->u1.constantValue, pExpr->u2.constantValue );
 #endif
 
 	if( hasParent != 0 )
@@ -1160,7 +1186,8 @@ int mk_execute_expr_constant( MK_VM_STRUCT *vm, MK_NODE_EXPR *pExpr, int hasPare
 			break;
 
 		case MK_TYPE_NODE_EXPR_FLOAT:
-			// not supported yet.
+			result =
+				mk_vm_create_float_frame_item( vm, *( (MK_FLOAT*)&pExpr->u1.constantValue ) );
 			break;
 
 		case MK_TYPE_NODE_EXPR_STRING:
@@ -1193,7 +1220,7 @@ int mk_execute_expr_return( MK_VM_STRUCT *vm, MK_NODE_EXPR *pExpr, int hasParent
 {
 	int retCode = MK_VM_EXECUTE_EXPR_RETURN_NEXTSTEP;
 #ifdef _DEBUG
-	printf( "retrun\n" );
+	fprintf( stdout, "retrun\n" );
 #endif
 
 	if( hasParent != 0 )
@@ -1209,7 +1236,7 @@ static
 int mk_execute_expr_break( MK_VM_STRUCT *vm, MK_NODE_EXPR *pExpr, int hasParent )
 {
 #ifdef _DEBUG
-	printf( "break\n" );
+	fprintf( stdout, "break\n" );
 #endif
 
 	if( hasParent != 0 )
@@ -1222,7 +1249,7 @@ static
 int mk_execute_expr_continue( MK_VM_STRUCT *vm, MK_NODE_EXPR *pExpr, int hasParent )
 {
 #ifdef _DEBUG
-	printf( "continue\n" );
+	fprintf( stdout, "continue\n" );
 #endif
 
 	if( hasParent != 0 )
@@ -1246,7 +1273,7 @@ int mk_execute_expr_symbol( MK_VM_STRUCT *vm, MK_NODE_EXPR *pExpr, int hasParent
 		return mk_raise_invalidate_parent_error( vm );
 
 #ifdef _DEBUG
-	printf( "loadsym %s\n", pExpr->u1.symbolName );
+	fprintf( stdout, "loadsym %s\n", pExpr->u1.symbolName );
 #endif
 
 	// search local variables(and arguments).
@@ -1286,7 +1313,7 @@ int mk_execute_expr_multiple( MK_VM_STRUCT *vm, MK_NODE_EXPR *pExpr, int hasPare
 		&vm->localStack;
 
 #ifdef _DEBUG
-	printf( "multiple->\n" );
+	fprintf( stdout, "multiple->\n" );
 #endif
 
 	if( hasParent != 0 )
@@ -1314,11 +1341,11 @@ int mk_execute_expr_at_symbol( MK_VM_STRUCT *vm, MK_NODE_EXPR *pExpr, int hasPar
 	MK_CHAR *symbolName = pExpr->u1.symbolName;
 	MK_VM_FRAME_ITEM *result = NULL;
 	MK_VM_FRAME_ITEM *pOwner = ( hasParent == 0 ) ? 
-		pOwner = vm->pCurrentFrame->pThis : mk_vm_pop_stack( vm->localStack );
+		pOwner = vm->pCurrentFrame->pThis : mk_vm_pop_stack( vm->localStack ), *pTop = pOwner;
 	MK_CHAR *super = NULL;
 
 #ifdef _DEBUG
-	printf( "loadsym %s\n", pExpr->u1.symbolName );
+	fprintf( stdout, "loadsym %s\n", pExpr->u1.symbolName );
 #endif
 
 	if( MK_TYPE_ATTRIBUTE(pExpr->flags ) == MK_TYPE_NODE_EXPR_DBLATSYMBOL )
@@ -1344,7 +1371,7 @@ int mk_execute_expr_at_symbol( MK_VM_STRUCT *vm, MK_NODE_EXPR *pExpr, int hasPar
 			"", 
 			0, 
 			MK_ERROR_TYPE_VM_ERROR | 1, 
-			symbolName, pOwner->classTypeValue.typeName, NULL );
+			symbolName, pTop->classTypeValue.typeName, NULL );
 	else
 		mk_vm_push_stack( vm->localStack, result );
 	return MK_VM_EXECUTE_EXPR_RETURN_NEXTSTEP;
@@ -1353,24 +1380,21 @@ int mk_execute_expr_at_symbol( MK_VM_STRUCT *vm, MK_NODE_EXPR *pExpr, int hasPar
 static
 int mk_execute_expr_new( MK_VM_STRUCT *vm, MK_NODE_EXPR *pExpr, int hasParent )
 {
-	MK_CLASS *pObject = NULL;
-	MK_VM_FRAME_ITEM *pObjectInstance = NULL;
+	MK_CLASS *pObject = NULL, *pTarget = NULL;
 	MK_FUNCTION *pFunction = NULL;
 	int retCode = 0;
-	MK_VM_FRAME_ITEM *pOwner = NULL;
+	MK_VM_FRAME_ITEM *pOwner = NULL, *pInstance = NULL;
 	MK_CHAR *pObjectClassName = mk_get_symbol_name_ptr( vm, CLASS_INTERNAL_OBJECT );
+	unsigned int index = 0;
+	unsigned int size = 
+		( pExpr->u2.args != NULL ) ? mk_size_vector( pExpr->u2.args ) : 0;
+	int argResult = MK_VM_EXECUTE_EXPR_RETURN_NEXTSTEP;
 	
 #ifdef _DEBUG
-	printf( "new %s\n", pExpr->u1.symbolName );
+	fprintf( stdout, "new %s\n", pExpr->u1.symbolName );
 #endif
 
-	// call new function
-	mk_find_item_hashtable( vm->code->classes, pObjectClassName, (void**)&pObject );
-	pFunction = 
-		mk_vm_find_function( vm, pObject, pExpr->u1.symbolName, 0 );
-	mk_find_item_hashtable( vm->global, pObjectClassName, (void**)&pObjectInstance );
-
-
+	// find target class
 	if( hasParent == 0 )
 	{
 		pOwner = vm->pCurrentFrame->pThis;
@@ -1381,12 +1405,63 @@ int mk_execute_expr_new( MK_VM_STRUCT *vm, MK_NODE_EXPR *pExpr, int hasParent )
 		pOwner = mk_get_at_vector( 
 			vm->localStack, mk_size_vector( vm->localStack ) - 1 );
 	}
+	pTarget = 
+		mk_vm_get_class( vm, pOwner );
+	if( pTarget == NULL )
+		return MK_VM_EXECUTE_EXPR_THROW;	// set error object in mk_vm_get_class
+
+	// call new function
+	mk_find_item_hashtable( 
+		vm->code->classes, pObjectClassName, (void**)&pObject );
+	pFunction = 
+		mk_vm_find_function( vm, pObject, pExpr->u1.symbolName, 0 );
 
 	retCode = mk_vm_call_function( vm, pFunction );
-	
-	// todo: call constracter
+	if( retCode != MK_VM_EXECUTE_EXPR_RETURN_NEXTSTEP )
+		return retCode;
+	pInstance = mk_get_at_vector( vm->localStack,
+		mk_size_vector( vm->localStack ) - 1 );
+
+	// call initialize
+	pFunction = 
+		mk_vm_find_function( vm, 
+		pTarget, 
+		mk_get_symbol_name_ptr( vm, "initialize" ), 
+		mk_size_vector( pExpr->u2.args ) );
+	if( pFunction != NULL )
+	{
+		// todo : create default value
+		for( index = 0; index < size; index ++ )
+		{
+			MK_VM_FRAME_ITEM *arg = NULL;
+			MK_NODE_EXPR *pTarget = 
+				mk_get_at_vector( pExpr->u2.args, index );
+			argResult = 
+				mk_execute_expr( vm, pTarget, 0 );
+			if( argResult != MK_VM_EXECUTE_EXPR_RETURN_NEXTSTEP )
+				break;	// error
+		}
+		if( index == size )
+		{
+			retCode = mk_vm_call_function( vm, pFunction );
+			if( retCode == MK_VM_EXECUTE_EXPR_RETURN_NEXTSTEP )
+			{
+				mk_vm_pop_stack( vm->localStack );
+				mk_vm_push_stack( vm->localStack, pInstance );
+			}
+			return retCode;
+		}
+		else
+		{
+			return MK_VM_EXECUTE_EXPR_THROW;	// todo: create exception objec
+		}
+	}
+	else
+	{
+		vm->exceptionObject = NULL;	// clear function not found exception.
+		return MK_VM_EXECUTE_EXPR_RETURN_NEXTSTEP;
+	}
 
-	return retCode;
 }
 
 static
@@ -1401,7 +1476,7 @@ int mk_execute_expr_call_function( MK_VM_STRUCT *vm, MK_NODE_EXPR *pExpr, int ha
 	int argResult = MK_VM_EXECUTE_EXPR_RETURN_NEXTSTEP;
 
 #ifdef _DEBUG
-	printf( "call %s\n", pExpr->u1.symbolName );
+	fprintf( stdout, "call %s\n", pExpr->u1.symbolName );
 #endif
 
 	if( hasParent == 0 )
@@ -1443,3 +1518,55 @@ int mk_execute_expr_call_function( MK_VM_STRUCT *vm, MK_NODE_EXPR *pExpr, int ha
 	else
 		return MK_VM_EXECUTE_EXPR_THROW;	// todo: create exception object
 }
+
+int mk_execute_expr_super_call( MK_VM_STRUCT *vm, MK_NODE_EXPR *pExpr, int hasParent )
+{
+	MK_VM_FRAME_ITEM *pOwner = NULL;
+	MK_CLASS *pClass = NULL;
+	MK_FUNCTION *pFunction = NULL;
+	unsigned int index = 0;
+	unsigned int size = 
+		( pExpr->u2.args != NULL ) ? mk_size_vector( pExpr->u2.args ) : 0;
+	int argResult = MK_VM_EXECUTE_EXPR_RETURN_NEXTSTEP;
+
+	if( hasParent == 0 )
+		pOwner = vm->pCurrentFrame->pThis;
+	else
+		pOwner = mk_vm_pop_stack( vm->localStack );
+
+	pOwner = 
+		mk_vm_get_super_class_instance( vm, pOwner );
+	if( pOwner == NULL )
+		;	// todo: not found super class.
+	mk_vm_push_stack( vm->localStack, pOwner );
+
+	pClass =
+		mk_vm_get_class( vm, pOwner );
+	if( pClass == NULL )
+		return MK_VM_EXECUTE_EXPR_THROW;	// set error object in mk_vm_get_class
+
+	pFunction = 
+		mk_vm_find_function( vm, 
+		pClass, 
+		vm->pCurrentFrame->pFunction->name, 
+		mk_size_vector( pExpr->u2.args ) );
+	if( pFunction == NULL )
+		return MK_VM_EXECUTE_EXPR_THROW;	// set error object in mk_vm_find_function
+
+	// todo : create default value
+	for( index = 0; index < size; index ++ )
+	{
+		MK_VM_FRAME_ITEM *arg = NULL;
+		MK_NODE_EXPR *pTarget = 
+			mk_get_at_vector( pExpr->u2.args, index );
+		argResult = 
+			mk_execute_expr( vm, pTarget, 0 );
+		if( argResult != MK_VM_EXECUTE_EXPR_RETURN_NEXTSTEP )
+			break;	// error
+	}
+	if( index == size )
+		return mk_vm_call_function( vm, pFunction );
+	else
+		return MK_VM_EXECUTE_EXPR_THROW;	// todo: create exception object
+
+}
